//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//


// ****************************************************************************
// FILE: tinyclock.cpp
// ABTRACT: Main implementation file for the today item
// ****************************************************************************
//

#include "stdafx.h"

// definitions

#define PXSPERLINES				20		// number of pixels per today lines

#define DISPTYPE_TIME			0
#define DISPTYPE_DATE			1
#define DISPTYPE_TIMEANDDATE	2
#define DISPTYPE_DATEANDTIME	3

// registry

#define REGISTRY_ROOT			TEXT("Software\\yasuho\\TinyClock")
#define VALUE_USEFONT           TEXT("UseFont")
#define VALUE_LOGFONT           TEXT("LogFont")
#define VALUE_USECOLOR          TEXT("UseColor")
#define VALUE_TEXTCOLOR         TEXT("TextColor")
#define VALUE_CUSTCOLORS        TEXT("CustColors")
#define VALUE_DISPTYPE			TEXT("DispType")

// global variables    

HICON               g_hIcon;
HINSTANCE           g_hInst;
HWND                g_hWnd;
TCHAR				g_szAppName[MAX_PATH];

BOOL				g_bInitialized;
SYSTEMTIME			g_lasttime;		// time last updated
INT					g_nLines;		// # of lines in today item
BOOL				g_bFontUpdated;	// logfont has been updated so lines should be recalculated

// get/set from/to registry

BOOL				g_usefont;
LOGFONT				g_logfont;
BOOL				g_usecolor;
COLORREF			g_textcolor;
COLORREF			g_custcolors[16];
INT					g_disptype;

// forward function declarations
static INT InitializeClasses();
LRESULT WINAPI CustomItemOptionsDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

/************************************************************************/
/* Compute today lines from logfont                                     */
/************************************************************************/
static void ComputeTodayLines(HWND hwnd)
{
	if (!g_bFontUpdated)
		return;

	HDC hDC = GetDC(hwnd);
	HFONT hFont = CreateFontIndirect(&g_logfont);
    HFONT hFontOld = (HFONT)SelectObject(hDC, hFont);

	TEXTMETRIC tm;

	GetTextMetrics(hDC, &tm);
	g_nLines = (tm.tmHeight + DRA::SCALEY(PXSPERLINES) - 1) / DRA::SCALEY(PXSPERLINES);
	if (g_nLines <= 0)
	{
		g_nLines = 1;	// at least it should be greater than 0
	}

    SelectObject(hDC, hFontOld);
    DeleteObject(hFont);
	ReleaseDC(hwnd, hDC);

	g_bFontUpdated = FALSE;
}

/************************************************************************/
/* Save settings to registry                                          */
/************************************************************************/
static BOOL SaveSettingsToRegistry()
{
    BOOL fret = FALSE;
    LONG lret;
    HKEY hkey = NULL;

    lret = RegCreateKeyEx(HKEY_CURRENT_USER, REGISTRY_ROOT, 0, NULL, 0, KEY_ALL_ACCESS, NULL, &hkey, NULL);
    if (lret == ERROR_SUCCESS)
    {
        lret = RegSetValueEx(hkey, VALUE_USEFONT, 0, REG_DWORD, (LPBYTE)&g_usefont, (DWORD)sizeof g_usefont);
        if (lret != ERROR_SUCCESS)
        {
            goto out;
        }

        lret = RegSetValueEx(hkey, VALUE_LOGFONT, 0, REG_BINARY, (LPBYTE)&g_logfont, (DWORD)sizeof g_logfont);
		if (lret != ERROR_SUCCESS)
        {
	        goto out;
		}

        lret = RegSetValueEx(hkey, VALUE_USECOLOR, 0, REG_DWORD, (LPBYTE)&g_usecolor, (DWORD)sizeof g_usecolor);
        if (lret != ERROR_SUCCESS)
        {
            goto out;
        }

        lret = RegSetValueEx(hkey, VALUE_TEXTCOLOR, 0, REG_DWORD, (LPBYTE)&g_textcolor, (DWORD)sizeof g_textcolor);
        if (lret != ERROR_SUCCESS)
        {
            goto out;
        }

        lret = RegSetValueEx(hkey, VALUE_CUSTCOLORS, 0, REG_BINARY, (LPBYTE)g_custcolors, (DWORD)sizeof g_custcolors);
        if (lret != ERROR_SUCCESS)
        {
            goto out;
        }

        lret = RegSetValueEx(hkey, VALUE_DISPTYPE, 0, REG_DWORD, (LPBYTE)&g_disptype, (DWORD)sizeof g_disptype);
        if (lret != ERROR_SUCCESS)
        {
            goto out;
        }

		fret = TRUE;
    }

out:
	if (hkey) RegCloseKey(hkey);

    return fret;
}

/************************************************************************/
/* Load settings from registry                                          */
/************************************************************************/
static BOOL LoadSettingsFromRegistry()
{
    BOOL fret = FALSE;
    LONG lret;
    HKEY hkey = NULL;
    DWORD size;

    lret = RegOpenKeyEx(HKEY_CURRENT_USER, REGISTRY_ROOT, 0, KEY_READ, &hkey);
    if (lret == ERROR_SUCCESS)
    {
        size = sizeof(g_usefont);
        lret = RegQueryValueEx(hkey, VALUE_USEFONT, 0, NULL, (LPBYTE)&g_usefont, &size);
        if (lret != ERROR_SUCCESS)
        {
            goto out;
        }

        size = sizeof(g_logfont);
	    lret = RegQueryValueEx(hkey, VALUE_LOGFONT, 0, NULL, (LPBYTE)&g_logfont, &size);
		if (lret != ERROR_SUCCESS)
        {
	        goto out;
		}
		g_bFontUpdated = TRUE;

        size = sizeof(g_usecolor);
        lret = RegQueryValueEx(hkey, VALUE_USECOLOR, 0, NULL, (LPBYTE)&g_usecolor, &size);
        if (lret != ERROR_SUCCESS)
        {
            goto out;
        }

        size = sizeof(g_textcolor);
        lret = RegQueryValueEx(hkey, VALUE_TEXTCOLOR, 0, NULL, (LPBYTE)&g_textcolor, &size);
        if (lret != ERROR_SUCCESS)
        {
            goto out;
        }

        size = sizeof(g_custcolors);
        lret = RegQueryValueEx(hkey, VALUE_CUSTCOLORS, 0, NULL, (LPBYTE)g_custcolors, &size);
        if (lret != ERROR_SUCCESS)
        {
            goto out;
        }

        size = sizeof(g_disptype);
        lret = RegQueryValueEx(hkey, VALUE_DISPTYPE, 0, NULL, (LPBYTE)&g_disptype, &size);
        if (lret != ERROR_SUCCESS)
        {
            goto out;
        }

		fret = TRUE;
    }

out:
	if (hkey) RegCloseKey(hkey);

    return fret;
}

/************************************************************************/
/* Load default logfont													*/
/************************************************************************/
static void LoadDefaultLogfont()
{
	HFONT hSysFont = (HFONT) GetStockObject(SYSTEM_FONT);
	GetObject(hSysFont, sizeof(LOGFONT), &g_logfont);

	g_bFontUpdated = TRUE;
}

/************************************************************************/
/* Initialize font information											*/
/************************************************************************/
static void InitializeFontInfo()
{
	if (!g_usefont)
	{
		LoadDefaultLogfont();
	}
}

/************************************************************************/
/* Initialization														*/
/************************************************************************/
static void Initialize()
{
	if (g_bInitialized)
		return;

	g_usefont = FALSE;
	memset(&g_logfont, 0, sizeof g_logfont);
	g_usecolor = FALSE;
	g_textcolor = 0;
	memset(g_custcolors, 0, sizeof g_custcolors);
	g_disptype = DISPTYPE_TIME;

	g_bFontUpdated = TRUE;

	LoadSettingsFromRegistry();

	InitializeFontInfo();

	g_bInitialized = TRUE;
}

/************************************************************************/
/* DeInitialize															*/
/************************************************************************/
static void Deinitialize()
{
}

/*************************************************************************/
/* Entry point for the dll                                                 */
/*************************************************************************/
BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
    switch (ul_reason_for_call)
    {
    case DLL_PROCESS_ATTACH:            
        
        // The DLL is being loaded for the first time by a given process.
        // Perform per-process initialization here.  If the initialization
        // is successful, return TRUE; if unsuccessful, return FALSE.
        g_hInst = (HINSTANCE)hModule;
        
        //load the icon
        g_hIcon = (HICON)LoadImage(g_hInst, MAKEINTRESOURCE(IDI_DISPLAYICON), IMAGE_ICON, DRA::SCALEX(16), DRA::SCALEY(16) ,LR_DEFAULTCOLOR);
        
		//initilize the application class, and set the global window handle
		LoadString(g_hInst, IDS_TODAY_STORAGE_APPNAME, g_szAppName, sizeof(g_szAppName) / sizeof(TCHAR));
        UnregisterClass(g_szAppName, g_hInst);
        InitializeClasses();
        g_hWnd = 0;
        break;
        
    case DLL_PROCESS_DETACH:
        // The DLL is being unloaded by a given process.  Do any
        // per-process clean up here, such as undoing what was done in
        // DLL_PROCESS_ATTACH.    The return value is ignored.
        
        DestroyIcon(g_hIcon);
        
        UnregisterClass(g_szAppName, g_hInst);
        g_hInst = NULL;
        break;           
    }
    
    return TRUE;
}

/*************************************************************************/
/* Handle any messages that may be needed for the plugin                 */
/* Handled messages:                                                     */
/*        WM_TODAYCUSTOM_QUERYREFRESHCACHE                                 */
/*        WM_CREATE                                                         */
/*        WM_LBUTTONUP                                                     */
/*        WM_PAINT                                                         */
/*        WM_DESTROY                                                         */
/*************************************************************************/
LRESULT CALLBACK WndProc (HWND hwnd, UINT uimessage, WPARAM wParam, LPARAM lParam) 
{
	SYSTEMTIME curtime;

	switch (uimessage)
    {          
        //check to see if a refresh is required
    case WM_TODAYCUSTOM_QUERYREFRESHCACHE: 
        {
            TODAYLISTITEM *ptliItem;
            INT iItemHeight;
            
            BOOL    bReturn = FALSE;
            
            // get the pointer to the item from the Today screen
            ptliItem = (TODAYLISTITEM*)wParam;

            if ((NULL == ptliItem) || (WaitForSingleObject(SHELL_API_READY_EVENT, 0) == WAIT_TIMEOUT))
            {
				return FALSE;
            }

			// check updates
			GetLocalTime(&curtime);
			if (curtime.wHour != g_lasttime.wHour || curtime.wMinute != g_lasttime.wMinute)
			{
				bReturn = TRUE;
			}

            // adjust the height of the today item based on text height
			ComputeTodayLines(hwnd);
            iItemHeight = DRA::SCALEY(PXSPERLINES * g_nLines);
			if (iItemHeight <= 0)
			{
				iItemHeight = 1;	// for safety
			}

			if (0 == ptliItem->cyp)
            {
                ptliItem->cyp = iItemHeight;
                bReturn = TRUE;
            }
            
            return bReturn;
        }        
        
    case WM_CREATE:         
		Initialize();
        break;
        
    case WM_LBUTTONUP: 
		// DialogBox(g_hInst, MAKEINTRESOURCE(IDD_TODAY_CUSTOM), hwnd, (DLGPROC)CustomItemOptionsDlgProc);
        break;          

	case WM_PAINT: 
        PAINTSTRUCT     ps;
        RECT            rcWindBounds; 
        RECT            rcMyBounds; 
        HDC             hDC;
        HFONT           hFontOld;
        COLORREF        crText;
        TCHAR           szTimeBuffer[32];

        GetWindowRect( hwnd, &rcWindBounds); 
        
        hDC = BeginPaint(hwnd, &ps);

		// create a custom rectangle relative to the client area
        rcMyBounds.left = 0;
        rcMyBounds.top = DRA::SCALEY(2);
        rcMyBounds.right = rcWindBounds.right - rcWindBounds.left;
        rcMyBounds.bottom = rcWindBounds.bottom - rcWindBounds.top;          
        
        // draw the icon on the screen
        SetBkMode(hDC, TRANSPARENT);
        DrawIcon(hDC, 2, 0, g_hIcon);
        
        HFONT hFont;

        // create the font
        hFont = CreateFontIndirect(&g_logfont);
    
        // Select the system font into the device context
        hFontOld = (HFONT) SelectObject(hDC, hFont);

		// text color
		if (g_usecolor)
		{
			crText = g_textcolor;
		}
		else
		{
	        // determine the theme's current text color
		    //  this color will change when the user picks a new theme,
			//  so get it each time the display needs to be painted
			crText = SendMessage(GetParent(hwnd), TODAYM_GETCOLOR, (WPARAM) TODAYCOLOR_TEXT, NULL);
		}

        SetTextColor(hDC, crText);

        // current time
		GetLocalTime(&curtime);
		g_lasttime = curtime;

		// format time/date
		switch (g_disptype)
		{
		default:
			wsprintf(szTimeBuffer, TEXT("%02d:%02d"), curtime.wHour, curtime.wMinute);
			break;
		case DISPTYPE_DATE:
			wsprintf(szTimeBuffer, TEXT("%2d/%2d"), curtime.wMonth, curtime.wDay);
			break;
		case DISPTYPE_TIMEANDDATE:
			wsprintf(szTimeBuffer, TEXT("%02d:%02d %2d/%2d"), curtime.wHour, curtime.wMinute, curtime.wMonth, curtime.wDay);
			break;
		case DISPTYPE_DATEANDTIME:
			wsprintf(szTimeBuffer, TEXT("%2d/%2d %02d:%02d"), curtime.wMonth, curtime.wDay, curtime.wHour, curtime.wMinute);
			break;
		}

		// draw the time
        rcMyBounds.left = rcMyBounds.left + DRA::SCALEX(28);
        DrawText(hDC, szTimeBuffer, -1, &rcMyBounds, DT_LEFT);
            
        // cleanup
        SelectObject(hDC, hFontOld);

        DeleteObject(hFont);

        EndPaint(hwnd, &ps);
        return 0;
        
    case WM_DESTROY :  
		Deinitialize();
        return 0 ;

    // this fills in the background with the today screen image
    case WM_ERASEBKGND:
        TODAYDRAWWATERMARKINFO dwi;
        dwi.hdc = (HDC)wParam;
        GetClientRect(hwnd, &dwi.rc);
        
        dwi.hwnd = hwnd;
        SendMessage(GetParent(hwnd), TODAYM_DRAWWATERMARK, 0,(LPARAM)&dwi);
        return TRUE;
    }

    return DefWindowProc (hwnd, uimessage, wParam, lParam) ;
}


/*************************************************************************/
/* Create and register our window class for the today item                 */
/*************************************************************************/
INT InitializeClasses()
{
    WNDCLASS         wc; 
    memset(&wc, 0, sizeof(wc));
    
    wc.style         = 0;                   
    wc.lpfnWndProc     = (WNDPROC) WndProc;
    wc.hInstance     = g_hInst;
    wc.hIcon         = 0;
    wc.hCursor         = 0;
    wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszClassName = g_szAppName;
    
    //register our window
    if(!RegisterClass(&wc))
    { 
        return 0 ;
    }
    return 1;
}

/*************************************************************************/
/* Initialize the DLL by creating a new window                             */
/*************************************************************************/
HWND InitializeCustomItem(TODAYLISTITEM *ptli, HWND hwndParent) 
{
    LPCTSTR appName = g_szAppName;
    
    //create a new window
    g_hWnd = CreateWindow(appName,appName,WS_VISIBLE | WS_CHILD,
        CW_USEDEFAULT,CW_USEDEFAULT,240,0,hwndParent, NULL, g_hInst, NULL) ;
    
    // attach our winproc to the newly created window
    SetWindowLong(g_hWnd, GWL_WNDPROC, (LONG) WndProc);
    
    //display the window
    ShowWindow (g_hWnd, SW_SHOWNORMAL);
    UpdateWindow (g_hWnd) ;  
    
	return g_hWnd;
}

/*************************************************************************/
/* Update font button information                                        */
/*************************************************************************/
void UpdateFontButtonInfo(HWND hDlg)
{
	if (g_usefont)
	{
		SendMessage(GetDlgItem(hDlg, IDC_USEFONT), BM_SETCHECK, BST_CHECKED, 0);
		EnableWindow(GetDlgItem(hDlg, IDC_FONT), TRUE);
	}
	else
	{
		SendMessage(GetDlgItem(hDlg, IDC_USEFONT), BM_SETCHECK, BST_UNCHECKED, 0);
		EnableWindow(GetDlgItem(hDlg, IDC_FONT), FALSE);
	}
}

/*************************************************************************/
/* Update color button information                                       */
/*************************************************************************/
void UpdateColorButtonInfo(HWND hDlg)
{
	if (g_usecolor)
	{
		SendMessage(GetDlgItem(hDlg, IDC_USECOLOR), BM_SETCHECK, BST_CHECKED, 0);
		EnableWindow(GetDlgItem(hDlg, IDC_COLOR), TRUE);
	}
	else
	{
		SendMessage(GetDlgItem(hDlg, IDC_USECOLOR), BM_SETCHECK, BST_UNCHECKED, 0);
		EnableWindow(GetDlgItem(hDlg, IDC_COLOR), FALSE);
	}
}

/*************************************************************************/
/* Update display type information                                       */
/*************************************************************************/
void UpdateDispTypeInfo(HWND hDlg)
{
	switch (g_disptype)
	{
	case DISPTYPE_TIME:
		SendMessage(GetDlgItem(hDlg, IDC_DISPTIME), BM_SETCHECK, BST_CHECKED, 0);
		break;
	case DISPTYPE_DATE:
		SendMessage(GetDlgItem(hDlg, IDC_DISPDATE), BM_SETCHECK, BST_CHECKED, 0);
		break;
	case DISPTYPE_TIMEANDDATE:
		SendMessage(GetDlgItem(hDlg, IDC_DISPTIMEANDDATE), BM_SETCHECK, BST_CHECKED, 0);
		break;
	case DISPTYPE_DATEANDTIME:
		SendMessage(GetDlgItem(hDlg, IDC_DISPDATEANDTIME), BM_SETCHECK, BST_CHECKED, 0);
		break;
	}
}

/*************************************************************************/
/* Message Handler for the options dialog                                 */
/*************************************************************************/
LRESULT WINAPI CustomItemOptionsDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static BOOL bUpdate;

    switch (message)
    {
    case WM_INITDIALOG:
        {
            SHINITDLGINFO shidi;

            // Create a Done button and size it.  
            shidi.dwMask = SHIDIM_FLAGS;
            shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
            shidi.hDlg = hDlg;
            SHInitDialog(&shidi);

			Initialize();

			UpdateFontButtonInfo(hDlg);
			UpdateColorButtonInfo(hDlg);
			UpdateDispTypeInfo(hDlg);

			bUpdate = FALSE;
        }
        return TRUE; 
        
    case WM_COMMAND:
        switch (LOWORD(wParam))
        {
		case IDOK:
			if (bUpdate)
				SaveSettingsToRegistry();

            EndDialog(hDlg, LOWORD(wParam));
            return TRUE;

		case IDC_USEFONT:
			g_usefont = !g_usefont;
			UpdateFontButtonInfo(hDlg);
			bUpdate = TRUE;
			break;

		case IDC_FONT:
			CHOOSEFONT cf;
			LOGFONT lf;

			lf = g_logfont;
			memset(&cf, 0, sizeof cf);
			cf.lStructSize = sizeof cf;
			cf.hInstance = g_hInst;
			cf.hwndOwner = hDlg;
			cf.lpLogFont = &lf;
			cf.Flags = CF_SCREENFONTS | CF_INITTOLOGFONTSTRUCT;
			if (!ChooseFont(&cf))
				break;

			g_logfont = lf;
			g_bFontUpdated = TRUE;

			bUpdate = TRUE;
			break;

		case IDC_USECOLOR:
			g_usecolor = !g_usecolor;
			UpdateColorButtonInfo(hDlg);
			bUpdate = TRUE;
			break;

		case IDC_COLOR:
			CHOOSECOLOR cc;

			memset(&cc, 0, sizeof cc);
			cc.lStructSize = sizeof cc;
			cc.hInstance = g_hInst;
			cc.hwndOwner = hDlg;
			cc.rgbResult = g_textcolor;
			cc.lpCustColors = g_custcolors;
			cc.Flags = CC_RGBINIT | CC_ANYCOLOR;
			if (!ChooseColor(&cc))
				break;

			g_textcolor = cc.rgbResult;
			bUpdate = TRUE;
			break;

		case IDC_DISPTIME:
			g_disptype = DISPTYPE_TIME;
			UpdateDispTypeInfo(hDlg);
			bUpdate = TRUE;
			break;

		case IDC_DISPDATE:
			g_disptype = DISPTYPE_DATE;
			UpdateDispTypeInfo(hDlg);
			bUpdate = TRUE;
			break;

		case IDC_DISPTIMEANDDATE:
			g_disptype = DISPTYPE_TIMEANDDATE;
			UpdateDispTypeInfo(hDlg);
			bUpdate = TRUE;
			break;

		case IDC_DISPDATEANDTIME:
			g_disptype = DISPTYPE_DATEANDTIME;
			UpdateDispTypeInfo(hDlg);
			bUpdate = TRUE;
			break;
        }
        break;

    case WM_DESTROY:
        break;

    }

    return FALSE;
}


// vi: ts=4 sw=4
